/******************************************************************************
*******************************************************************************
**
**  Copyright (C) 2005 Red Hat, Inc.  All rights reserved.
**
**  This copyrighted material is made available to anyone wishing to use,
**  modify, copy, or redistribute it subject to the terms and conditions
**  of the GNU General Public License v.2.
**
*******************************************************************************
******************************************************************************/

#ifndef _BLOCK_LIST_H
#define _BLOCK_LIST_H

#include "bitmap.h"

/* Must be kept in sync with mark_to_bitmap array in block_list.c */
enum mark_block {
	block_free = 0,
	block_used = 1,
	indir_blk = 2,
	inode_dir = 3,
	inode_file = 4,
	inode_lnk = 5,
	inode_blk = 6,
	inode_chr = 7,
	inode_fifo = 8,
	inode_sock = 9,
	leaf_blk = 10,
	meta_other = 11,
	meta_free = 12,
	meta_eattr = 13,
	bad_block = 14,	/* Contains at least one bad block */
	meta_inval = 15,
	/* above this are nibble-values 0x0-0xf */
	/* A block can be both duplicate and one of the above. */
	dup_block = 16,	/* Contains at least one duplicate block */
	eattr_block = 17,	/* Contains an eattr */
};

struct block_query {
	uint8_t block_type;
	uint8_t dup_block;
	uint8_t eattr_block;
};

/* bitmap implementation */
struct block_list {
	struct bmap group_map;
	struct bmap dup_map;
	struct bmap eattr_map;
};

struct block_list *block_list_create(uint64_t size);
int block_mark(struct block_list *il, uint64_t block, enum mark_block mark);
int block_set(struct block_list *il, uint64_t block, enum mark_block mark);
int block_unmark(struct block_list *il, uint64_t block, enum mark_block m);
int block_clear(struct block_list *il, uint64_t block);
int block_check(struct block_list *il, uint64_t block,
		struct block_query *val);
int block_check_for_mark(struct block_list *il, uint64_t block,
			 enum mark_block mark);
void *block_list_destroy(struct block_list *il);
int find_next_block_type(struct block_list *il, enum mark_block m, uint64_t *b);

#endif /* _BLOCK_LIST_H */
